import markdown2 as md
from django import template
from django.template.defaultfilters import stringfilter
from django.utils.safestring import mark_safe

register = template.Library()


@register.filter(name='markdown')
@stringfilter
def to_markdown(value: str) -> str:
    """
    Transforms a Markdown text into HTML.
    """

    classes_dict = {}
    markdowner = md.Markdown(extras={
        'footnotes': None,
        'header-ids': None,
        'fenced-code-blocks': None,
        'html-classes': classes_dict,
    })

    html = mark_safe(markdowner.convert(value))
    return html



# -*- coding: utf-8 -*-
VERSION = (0, 0, 3)


def get_version(version: tuple) -> str:
    """
    Gets the version of the package based on a :class:`tuple` and returns a :class:`str`.
    This method is based on ``django-extensions`` get_version method.
    """

    if not isinstance(version[-1], int):
        str_version = '.'.join(list(map(str, version[:-1]))) + '_{}'.format(version[-1])
        return str_version

    str_version = '.'.join(list(map(str, version)))
    return str_version


__version__ = get_version(VERSION)



from django.conf import settings


def pytest_configure():
    settings.configure(
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
            },
        },
    )



import pathlib
import shutil

import pytest

from django_markdowner.views import MarkdownView


@pytest.fixture(scope='module', autouse=True)
def copy_test_md(md_file: str = './test_md/test.md'):
    """
    This module copies a Test Markdown file.
    """

    if md_file.startswith('./'):
        md_folder = pathlib.Path(__file__).parent
        md_file = md_folder / md_file
    else:
        md_file = pathlib.Path(md_file)
        md_folder = md_file.parent

    if not md_file.exists() and not md_file.is_file():
        pytest.fail(msg='Not markdown file found.')

    template_md = pathlib.Path('./django_markdowner/markdown/')
    template_folder = pathlib.Path('./django_markdowner/templates/')

    # Creates folders
    if not template_md.exists():
        template_md.mkdir()
    if not template_folder.exists():
        template_folder.mkdir()

    # Copies file
    shutil.copy(md_file, template_md)
    template_file = template_folder / pathlib.Path('%s.html' % md_file.stem)
    if not template_file.exists():
        template_file.touch()

    yield

    # Deletes files
    shutil.rmtree(template_md)
    shutil.rmtree(template_folder)


class MarkdownViewForTesting(MarkdownView):
    """
    Test view just to check if everything works correctly.
    """

    template_name = 'test_md/test.html'
    markdown_name = 'test_md/test.md'


def test_testing(rf):
    """
    Checks if markdown was correctly formatted.
    """

    request = rf.get('/')
    view = MarkdownViewForTesting.as_view()(request)
    # Checks for context
    assert all([key in view.context_data for key in ['md', 'raw_md']])



import inspect
import pathlib

from django.core.exceptions import ImproperlyConfigured
from django.views.generic import TemplateView

from .templatetags import md_tags


class MarkdownView(TemplateView):
    """
    Renders a Markdown file into a Django Template.
    """

    markdown_name = None

    def get_context_data(self, **kwargs):
        """
        Adds markdown formatted to the context as `md`.
        """

        context = super().get_context_data(**kwargs)
        context['raw_md'] = self.get_markdown_text()
        context['md'] = self.parse_markdown()
        return context

    def get_markdown_names(self) -> list:
        """
        Returns a list of markdown files to be used for the request.
        """

        if self.markdown_name is None:
            raise ImproperlyConfigured("'markdown_name' is not defined.")
        return [self.markdown_name]

    def get_markdown_files(self, md_file: str) -> pathlib.Path:
        """
        Returns the file of the markdown as :class:`pathlib.Path`.
        """

        folder = pathlib.Path(inspect.getfile(self.__class__)).parent
        md_file = folder / md_file

        if not md_file.exists() or not md_file.is_file():
            raise ImproperlyConfigured("One of the Markdown files does not exist or is not a file.")

        return md_file

    def get_markdown_text(self) -> str:
        """
        Gets markdown text.
        """

        md_files = list(map(self.get_markdown_files, self.get_markdown_names()))
        markdown_text = ''

        for md_file in md_files:
            markdown_text += md_file.read_text() + '\n'

        return markdown_text

    def parse_markdown(self) -> str:
        """
        Transform Raw Markdown to HTML.
        """

        return md_tags.to_markdown(self.get_markdown_text())



# -*- coding: utf-8 -*-
"""
Based on Django's ``setup.py``.
"""

import pathlib

from setuptools import setup, find_packages


# Gets version from django_extensions package
version = __import__('django_markdowner').__version__

# List of required packages
install_requires = ['django>=2', 'markdown2']

# List of keywords
keywords = ['django', 'markdown']

# Long description
long_description = """"""
readme_path = pathlib.Path('README.rst')
# Gets long description from README.rst
if readme_path.exists() and readme_path.is_file():
    long_description = readme_path.read_text()

setup(
    name='django-markdowner',
    version=version,
    description='Markdown to Template builder for Django.',
    long_description=long_description,
    author='LeCuay',
    author_email='suso.becerra98@gmail.com',
    url='https://github.com/LeCuay/django-markdowner',
    license='MIT License',
    platforms=['any'],
    packages=find_packages(),
    install_requires=install_requires,
    project_urls={
        'Source Code': 'https://github.com/LeCuay/django-markdowner',
    },
    tests_require=[
        'Django',
        'Werkzeug',
        'pytest',
        'pytest-django',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Utilities',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
